#include "include/glad.h"
#include <GLFW/glfw3.h>
#include <iostream>

const char* vertexShaderSource = "#version 330 core\n"
	"layout (location = 0) in vec3 aPos;\n"
	"void main()\n"
	"{\n"
	"	gl_Position = vec4(aPos.x, aPos.y, aPos.z, 1.0);\n"
	"}\0";

const char* fragmentShaderSource = "#version 330 core\n"
	"out vec4 FragColor;\n"
	"void main()\n"
	"{\n"
	"	FragColor = vec4(1.0f, 0.5f, 0.2f, 1.0f);\n"
	"}\0";


void framebuffer_size_callback(GLFWwindow* window, int width, int height);
void processInput(GLFWwindow* window);
void compileShaders(unsigned int* shader, bool fragment);
void createShaderProgram(unsigned int* shaderProgram);
void setupGLFW(GLFWwindow** window);

/** SINGLE TRIANGLE VERTICES
float vertices[] = {
	-0.5f, -0.5f, 0.0f,
	0.5f, -0.5f, 0.0f,
	0.0f, 0.5f, 0.0f
};
*/

float verticesa[] = {
	//first triangle
	0.5f, 0.5f, 0.0f, // top right
	0.5f, -0.5f, 0.0f, // bottom right
	-0.5f, 0.5f, 0.0f, //top left
};
float verticesb[] = {
	0.5f, -0.5f, 0.0f, // bottom right
	-0.5f, -0.5f, 0.0f, // bottom left
	-0.5f, 0.5f, 0.0f // top left
};

/**
float vertices[] = {
	0.5f, 0.5f, 0.0f, // top right
	0.5f, -0.5f, 0.0f, // bottom right
	-0.5f, -0.5f, 0.0f, // bottom left
	-0.5f, 0.5f, 0.0f, // top left
};

unsigned int indices[] = { // note, starts from zero
	0, 1, 3, // first triangle
	1, 2, 3 // second triangle
};
**/

bool wireframe = false;

int main()
{
	GLFWwindow* window;
	unsigned int shaderProgram;
	setupGLFW(&window);


	createShaderProgram(&shaderProgram);
	
	unsigned int VAOa;
	glGenVertexArrays(1, &VAOa);

	unsigned int VBOa;
	glGenBuffers(1, &VBOa);

	unsigned int VAOb;
	glGenVertexArrays(1, &VAOb);

	unsigned int VBOb;
	glGenBuffers(1, &VBOb);



	// bind Vertex Array Object
	glBindVertexArray(VAOa);
	// copy our vertices into a buffer for opengl to use
	glBindBuffer(GL_ARRAY_BUFFER, VBOa);
	glBufferData(GL_ARRAY_BUFFER, sizeof(verticesa), verticesa, GL_STATIC_DRAW);
	// copy our indexes 
	
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*) 0);
	glEnableVertexAttribArray(0);

	// bind Vertex Array Object
	glBindVertexArray(VAOb);
	// copy our vertices into a buffer for opengl to use
	glBindBuffer(GL_ARRAY_BUFFER, VBOb);
	glBufferData(GL_ARRAY_BUFFER, sizeof(verticesb), verticesb, GL_STATIC_DRAW);
	// copy our indexes 
	
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*) 0);
	glEnableVertexAttribArray(0);



	while(!glfwWindowShouldClose(window)) {
		//input
		processInput(window);

		//rendering
		glClearColor(0.2f, 0.3f, 0.3f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT);
	

		glUseProgram(shaderProgram);
		glBindVertexArray(VAOa);
		glDrawArrays(GL_TRIANGLES, 0, 3);

		glBindVertexArray(VAOb);
		glDrawArrays(GL_TRIANGLES, 0, 3);

		glBindVertexArray(0); // unbinds the VAO so its ready to be bound again for the next render

		// check and call events and swap buffers
		glfwSwapBuffers(window);
		glfwPollEvents();
	}

	glfwTerminate();
	return 0;
}

void setupGLFW(GLFWwindow** window)
{
	glfwInit();
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	*window = glfwCreateWindow(800, 600, "LearnOpenGL", NULL, NULL);
	if (window == NULL) {
		std::cout << "Failed to create GLFW window\n";
		glfwTerminate();
		exit(-1);
	}
	glfwMakeContextCurrent(*window);

	if (!gladLoadGLLoader((GLADloadproc) glfwGetProcAddress)) {
		std::cout << "Failed to init GLAD\n";
		exit(-1);
	}

	glViewport(0, 0, 800, 600);
	glfwSetFramebufferSizeCallback(*window, framebuffer_size_callback);
}


void framebuffer_size_callback(GLFWwindow* window, int width, int height)
{
	glViewport(0, 0, width, height);
}

void processInput(GLFWwindow* window)
{
	if (glfwGetKey(window, GLFW_KEY_ESCAPE) == GLFW_PRESS)
		glfwSetWindowShouldClose(window, true);
	if (glfwGetKey(window, GLFW_KEY_ENTER) == GLFW_PRESS) // triggers wireframe mode
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	if (glfwGetKey(window, GLFW_KEY_BACKSPACE) == GLFW_PRESS) // disables wireframe mode
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

}

void compileShaders(unsigned int* shader, bool fragment, const char** source)
{
	int success;
	char infoLog[512];

	*shader = glCreateShader(fragment ? GL_FRAGMENT_SHADER : GL_VERTEX_SHADER);

	glShaderSource(*shader, 1, &(*source), NULL);

	glCompileShader(*shader);

	glGetShaderiv(*shader, GL_COMPILE_STATUS, &success);
	if (!success) {
		glGetShaderInfoLog(*shader, 512, NULL, infoLog);
		std::cout << "ERROR::SHADER::" << (fragment ? "FRAGMENT" : "VERTEX") << "::COMPILATION_FAILED\n" << infoLog << "\n";
	}
}

void createShaderProgram(unsigned int* shaderProgram) {
	int success;
	char infoLog[512];
	unsigned int vertexShader, fragmentShader;

	compileShaders(&vertexShader, false, &vertexShaderSource);
	compileShaders(&fragmentShader, true, &fragmentShaderSource);

	*shaderProgram = glCreateProgram();
	glAttachShader(*shaderProgram, vertexShader);
	glAttachShader(*shaderProgram, fragmentShader);
	glLinkProgram(*shaderProgram);

	glGetProgramiv(*shaderProgram, GL_LINK_STATUS, &success);
	if (!success) {
		glGetProgramInfoLog(*shaderProgram, 512, NULL, infoLog);
		std::cout << "ERROR::SHADER::LINK_FAILED\n" << infoLog << "\n";
	}

	glDeleteShader(vertexShader);
	glDeleteShader(fragmentShader);
}
